/*
 * Created on 2005. 3. 22
 */
package org.manalith.manager;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;


import org.apache.log4j.Logger;
import org.manalith.db.ConnectionFactory;
import org.manalith.resource.Article;
import org.manalith.resource.ArticleFormat;
import org.manalith.resource.User;


/**
 * @author setzer
 */
public class ArticleManager {
    private Connection conn;
    private static ArticleManager manager = null;
    private static Logger logger = Logger.getLogger(ArticleManager.class);
    
    private ArticleManager(){
        try {
            conn = ConnectionFactory.getConnection();
        } catch (SQLException ex) {
            logger.error("SQLException: " + ex.getMessage()); 
            logger.error("SQLState: " + ex.getSQLState()); 
            logger.error("VendorError: " + ex.getErrorCode()); 
        }
    }
    
    public static ArticleManager instance(){
        if(manager == null){
            manager = new ArticleManager();
        }
        return manager;
    }
    
    public void setup(){
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("CREATE TABLE manalith_blog_article(");
        sb.append("id SERIAL NOT NULL PRIMARY KEY,");
        sb.append("num INT NOT NULL,");
        sb.append("blogOwnerId VARCHAR(12) NOT NULL REFERENCES manalith_blog(owner),");
        sb.append("title VARCHAR(200) NOT NULL,");
        sb.append("author VARCHAR(12) NOT NULL REFERENCES manalith_member(id),");
        sb.append("category VARCHAR(50) NOT NULL,");
        sb.append("date TIMESTAMP NOT NULL DEFAULT now(),");
        sb.append("contents TEXT NOT NULL");
        sb.append("format VARCHAR(8) check(format in ('text','html')) NOT NULL,");
        sb.append("totalCommentCount int NOT NULL DEFAULT 0,");
        sb.append("totalTrackbackCount int NOT NULL DEFAULT 0");
        sb.append(")");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.executeUpdate();
        }catch(SQLException e){
            logger.error(e);
        }finally {
            if (pstmt != null) {
                try {
                    pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    public void createArticle(Article article){
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        int num;
        
        StringBuffer sb1 = new StringBuffer();
        sb1.append("SELECT MAX(num) + 1 AS num FROM manalith_blog_article ");
        sb1.append("WHERE blogOwnerId = ?");
        
        StringBuffer sb2 = new StringBuffer();
        sb2.append("INSERT INTO manalith_blog_article(blogOwnerId,title,author,category,date,contents,format,num) ");
        sb2.append("VALUES(?,?,?,?,?,?,?,?)");
        
        try{
            pstmt = conn.prepareStatement(sb1.toString());
            pstmt.setString(1,article.getBlogOwnerId());
            rs = pstmt.executeQuery();
            
            rs.next();
            num = rs.getInt("num");
            if(num == 0){
                num = 1;
            }

            pstmt = conn.prepareStatement(sb2.toString());
            pstmt.setString(1,article.getBlogOwnerId());
            pstmt.setString(2,article.getTitle());
            pstmt.setString(3,article.getAuthor().toString());
            pstmt.setString(4,article.getCategory());
            pstmt.setTimestamp(5,new Timestamp(article.getDate().getTime()));
            pstmt.setString(6,article.getContents());
            pstmt.setString(7,article.getFormat().toString());
            pstmt.setInt(8,num);
            
            pstmt.executeUpdate();
            
            increaseArticleCount(article.getBlogOwnerId());
        }catch(SQLException e){
            logger.error(e);
        }catch(ClassCastException e){
            logger.error(e);
        }finally {
            if (rs != null) {
                try {
                  rs.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                rs = null; 
            } 
            if (pstmt != null) {
                try {
                    pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
    }
    
    public Article getArticle(String blogOwnerId, int articleId){
        Article article = null;
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        String str = null;
        
        sb.append("SELECT num, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE id = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setInt(1,articleId);
            
            rs = pstmt.executeQuery();
            
            if(rs.next()){
                article = new Article();
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(ArticleCommentManager.instance().getComments(articleId));
        	      article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
            }
        }catch(SQLException e){
            logger.error(e.toString());
        }catch(Exception e){
            logger.error(e.toString());            
        }finally {
            if (rs != null) {
                try {
                  rs.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                rs = null; 
            } 
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return article;
    }
    
    public Article getArticle(int articleId){
        Article article = null;
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        String str = null;
        
        sb.append("SELECT num, blogOwnerId, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE id = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setInt(1,articleId);
            
            rs = pstmt.executeQuery();
            
            if(rs.next()){
                article = new Article();
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(rs.getString("blogOwnerId"));
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(ArticleCommentManager.instance().getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
        	      article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
            }
        }catch(SQLException e){
            logger.error(e);
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
        
        return article;
    }
    
//  TODO : trackbacks까지 함께 가져 올 것
    public List getArticles(String blogOwnerId){
        Article article = null;
        int articleId = 0;
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT num, id, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? ");
        sb.append("ORDER BY id DESC");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                article = new Article();
                articleId = rs.getInt("id");
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e.toString());
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
//  TODO : trackbacks까지 함께 가져 올 것
    public List getArticles(String blogOwnerId, String category){
        Article article = null;
        int articleId = 0;
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT id, num, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? AND category=?");
        sb.append("ORDER BY id DESC");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            pstmt.setString(2,category);
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                article = new Article();
                articleId = rs.getInt("id");
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e);
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
    public List getArticles(String blogOwnerId, int page, int totalArticleSize, int pageSize){
        Article article = null;
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT id, num, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=?");
        sb.append("ORDER BY id DESC LIMIT ? OFFSET ?");
        
        int previous = pageSize * (page - 1);
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            pstmt.setInt(2,pageSize);
            pstmt.setInt(3,previous);
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                article = new Article();
                int articleId = rs.getInt("id");
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e);
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
    public List getArticles(String blogOwnerId, Article article){
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT id, num, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? AND id=? ");
        sb.append("ORDER BY id DESC");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            pstmt.setInt(2,article.getId());
            rs = pstmt.executeQuery();
            
            if(rs.next()){
                article = new Article();
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(article.getId()));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(article.getId()));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e);
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
    public List getArticles(String blogOwnerId, java.util.Date date){
        Article article = null;
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        int articleId = 0;
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT id, num, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? AND date > ? AND date < ?");
        sb.append("ORDER BY id DESC");
        
        try{
            Calendar cl = Calendar.getInstance();
            cl.setTime(date);
            cl.add(Calendar.DATE,1);
            
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            pstmt.setDate(2,new Date(date.getTime()));
            pstmt.setDate(3,new Date(cl.getTime().getTime()));
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                article = new Article();
                articleId = rs.getInt("id");
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e.toString());
        }catch(Exception e){
            logger.error(e.toString());            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
    public List getRecentArticles(String blogOwnerId, int limitation){
        Article article = null;
        int articleId = 0;
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT num, id, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? ");
        sb.append("ORDER BY id DESC LIMIT ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            pstmt.setInt(2,limitation);
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                article = new Article();
                articleId = rs.getInt("id");
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e);
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
    public List getArticles(String blogOwnerId, User author){
        Article article = null;
        List articleList = new ArrayList();
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        int articleId = 0;
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT id, num, title, author, category, date, contents, format, totalCommentCount, totalTrackbackCount ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? AND author=?");
        sb.append("ORDER BY id DESC");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,blogOwnerId);
            pstmt.setString(2,author.toString());
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                article = new Article();
                articleId = rs.getInt("id");
                article.setId(articleId);
                article.setNum(rs.getInt("num"));
                article.setBlogOwnerId(blogOwnerId);
                article.setTitle(rs.getString("title"));
                article.setAuthor(new User(rs.getString("author")));
                article.setCategory(rs.getString("category"));
                article.setDate(new Date(rs.getTimestamp("date").getTime()));
                article.setContents(rs.getString("contents"));
                article.setFormat(new ArticleFormat(rs.getString("format")));
                article.setTotalCommentCount(rs.getInt("totalCommentCount"));
                article.setTotalTrackbackCount(rs.getInt("totalTrackbackCount"));
                article.setComments(commentManager.getComments(articleId));
                article.setTrackbacks(TrackbackManager.instance().getTrackbacks(articleId));
      	      		article.setFiles(FileManager.instance().getConnectedFiles(article.getId()));
                articleList.add(article);
            }
        }catch(SQLException e){
            logger.error(e);
        }catch(Exception e){
            logger.error(e);            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e);
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e);
                }
                pstmt = null; 
            } 
        }
        
        return articleList;
    }
    
    public int getMatchedArticleId(Article article){
        int articleId = 0;
        ArticleCommentManager commentManager = ArticleCommentManager.instance();
        
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        
        sb.append("SELECT id ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? AND title=? AND author=? AND category=? AND date=? AND contents=? AND format=? ");
        sb.append("ORDER BY id DESC");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.setString(1,article.getBlogOwnerId());
            pstmt.setString(2,article.getTitle());
            pstmt.setString(3,article.getAuthor().toString());
            pstmt.setString(4,article.getCategory());
            pstmt.setTimestamp(5,new Timestamp(article.getDate().getTime()));
            pstmt.setString(6,article.getContents());
            pstmt.setString(7,article.getFormat().toString());
            rs = pstmt.executeQuery();
            
            if(rs.next())
                articleId = rs.getInt("id");
            
        }catch(SQLException e){
            logger.error(e.toString());
        }catch(Exception e){
            logger.error(e.toString());            
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
        
        return articleId;
        
    }
    
    public void updateArticle(Article article){
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("UPDATE manalith_blog_article ");
        sb.append("SET title=?, category=?, date=?, contents=?, format=? ");
        sb.append("WHERE id = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1, article.getTitle());
            pstmt.setString(2, article.getCategory());
            pstmt.setTimestamp(3, new Timestamp(article.getDate().getTime()));
            pstmt.setString(4, article.getContents());
            pstmt.setString(5, article.getFormat().toString());
            pstmt.setInt(6, article.getId());
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            logger.error(e.toString());
        }catch(Exception e){
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    public void destroyArticle(String blogOwnerId, int articleId){
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("DELETE FROM manalith_blog_article ");
        sb.append("WHERE id = ?");
        
        try{
            ArticleCommentManager.instance().destroyAllComments(articleId);
            TrackbackManager.instance().deleteTrackbacks(articleId);
            
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setInt(1,articleId);
            
            pstmt.executeUpdate();
            
            decreaseArticleCount(blogOwnerId);
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    //COMMENT, TRACKBACK 등 부가 요소의 동시 제거
    public void destroyAllArticles(String blogOwnerId){
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("DELETE FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1,blogOwnerId);
            
            pstmt.executeUpdate();
            
            restoreArticleCount(blogOwnerId);
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    private void increaseArticleCount(String blogOwnerId) throws SQLException{
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("UPDATE manalith_blog ");
        sb.append("SET totalArticleCount = totalArticleCount + 1 ");
        sb.append("WHERE owner = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1, blogOwnerId);
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            throw e;
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    private void decreaseArticleCount(String blogOwnerId) throws SQLException{
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("UPDATE manalith_blog ");
        sb.append("SET totalArticleCount = totalArticleCount - 1 ");
        sb.append("WHERE owner = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1, blogOwnerId);
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            throw e;
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    private void restoreArticleCount(String blogOwnerId) throws SQLException{
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("UPDATE manalith_blog ");
        sb.append("SET totalArticleCount = 0 ");
        sb.append("WHERE owner = ?");
        
        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1, blogOwnerId);
            
            pstmt.executeUpdate();
        }catch(SQLException e){
            throw e;
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }
    
    public List getCategories(String userId){
        List categories = new ArrayList();
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        
        StringBuffer sb = new StringBuffer();
        sb.append("SELECT category ");
        sb.append("FROM manalith_blog_article ");
        sb.append("WHERE blogOwnerId=? ");
        sb.append("GROUP BY category");

        try{
            pstmt = conn.prepareStatement(sb.toString());
            
            pstmt.setString(1,userId);
            
            rs = pstmt.executeQuery();
            
            while(rs.next()){
                categories.add(rs.getString("category"));
            }
        }catch(SQLException e){
            logger.error(e.toString());
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
        
        return categories;
    }
}
