/*
 * Created on 2005. 3. 22
 */
package org.manalith.manager;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;


import org.apache.log4j.Logger;
import org.manalith.db.ConnectionFactory;
import org.manalith.resource.Article;
import org.manalith.resource.Blog;
import org.manalith.resource.Template;
import org.manalith.resource.User;

/**
 * @author setzer
 */
public class BlogManager {

    private Connection conn;
    private static BlogManager manager = null;
    private static Logger logger = Logger.getLogger(BlogManager.class);

    private BlogManager() {
        try {
            conn = ConnectionFactory.getConnection();
        } catch (SQLException ex) {
            logger.error("SQLException: " + ex.getMessage()); 
            logger.error("SQLState: " + ex.getSQLState()); 
            logger.error("VendorError: " + ex.getErrorCode()); 
        }
    }

    public static BlogManager instance() {
        if(manager == null){
            manager = new BlogManager();
        }
        return manager;
    }

    public void setup() {
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("CREATE TABLE manalith_blog(");
        sb.append("idx SERIAL NOT NULL,");
        sb.append("title VARCHAR(200) NOT NULL,");
        sb.append("description TEXT NOT NULL,");
        sb.append("template VARCHAR(20) DEFAULT 'default' NOT NULL,");
        sb.append("owner VARCHAR(12) NOT NULL PRIMARY KEY REFERENCES manalith_member(id),");
        sb.append("created TIMESTAMP NOT NULL DEFAULT now()");
        sb.append("totalArticleCount INT NOT NULL DEFAULT 0,");
        sb.append("pageSize INT NOT NULL DEFAULT 10 ");
        sb.append("url VARCHAR(200) NOT NULL ");
        sb.append("allowRSS BOOL NOT NULL DEFAULT TRUE,");
        sb.append("titleImage VARCHAR(50) NULL,");
        sb.append("backgroundImage VARCHAR(50) NULL");
        sb.append(");");

        try {
            pstmt = conn.prepareStatement(sb.toString());
            pstmt.executeUpdate();
        } catch (SQLException e) {
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }

    public boolean isBlogOwner(User user) {
        return isBlogOwner(user.getId());
    }

    public boolean isBlogOwner(String userId) {
        boolean result = false;

        PreparedStatement pstmt = null;
        ResultSet rs = null;
        String sql = "SELECT title FROM manalith_blog WHERE owner=?";

        try {
            pstmt = conn.prepareStatement(sql);

            pstmt.setString(1, userId);

            rs = pstmt.executeQuery();

            if (rs.next()) {
                result = true;
            }
        } catch (SQLException e) {
            logger.error(e.toString());
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }

        return result;
    }

    //FIXME : getAuthors() 이후 루핑하여 manalith_blog_author에 집어넣으며,
    // manalith_blog_category와 연결되어 있어야 한다.
    public void createBlog(Blog blog) {
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("INSERT INTO manalith_blog(title, description, template, owner, pageSize, url, allowRSS) ");
        sb.append("VALUES(?,?,?,?,?,?,?)");

        try {
            pstmt = conn.prepareStatement(sb.toString());

            pstmt.setString(1, blog.getTitle());
            pstmt.setString(2, blog.getDescription());
            pstmt.setString(3, blog.getTemplate().toString());
            pstmt.setString(4, blog.getOwner().getId());
            pstmt.setInt(5, blog.getPageSize());
            pstmt.setString(6, blog.getUrl());
            pstmt.setBoolean(7, blog.getAllowRSS());

            pstmt.executeUpdate();
        } catch (SQLException e) {
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }

    public Blog getBlog(String userId) {
        Blog blog = getBlogInfo(userId);
        if(blog != null){
	        ArticleManager articleManager = ArticleManager.instance();
	        BlogAuthorManager authorManager = BlogAuthorManager.instance();
	        BlogBookmarkManager bookmarkManager = BlogBookmarkManager.instance();
	        blog.setCategories(articleManager.getCategories(userId));
	        blog.setArticles(articleManager.getArticles(userId));
	        blog.setAuthors(authorManager.getAuthors(userId));
	        blog.setBookmarks(bookmarkManager.getBookmarks(userId));
        }
        return blog;
    }

    public Blog getBlog(String userId, String category) {
        Blog blog = getBlogInfo(userId);
        if(blog != null){
	        ArticleManager articleManager = ArticleManager.instance();
	        BlogAuthorManager authorManager = BlogAuthorManager.instance();
	        BlogBookmarkManager bookmarkManager = BlogBookmarkManager.instance();
	        blog.setCategories(articleManager.getCategories(userId));
	        blog.setArticles(articleManager.getArticles(userId, category));
	        blog.setAuthors(authorManager.getAuthors(userId));
	        blog.setBookmarks(bookmarkManager.getBookmarks(userId));
        }
        return blog;
    }
    
    public Blog getBlog(String userId, int page) {
        Blog blog = getBlogInfo(userId);
        if(blog != null){
	        ArticleManager articleManager = ArticleManager.instance();
	        BlogAuthorManager authorManager = BlogAuthorManager.instance();
	        BlogBookmarkManager bookmarkManager = BlogBookmarkManager.instance();
	        blog.setCategories(articleManager.getCategories(userId));
	        blog.setArticles(articleManager.getArticles(userId, page, blog.getTotalArticleCount(), blog.getPageSize()));
	        blog.setAuthors(authorManager.getAuthors(userId));
	        blog.setBookmarks(bookmarkManager.getBookmarks(userId));
        }
        return blog;
    }
    
    public Blog getBlog(String userId, Article article) {
        Blog blog = getBlogInfo(userId);
        if(blog != null){
	        ArticleManager articleManager = ArticleManager.instance();
	        BlogAuthorManager authorManager = BlogAuthorManager.instance();
	        BlogBookmarkManager bookmarkManager = BlogBookmarkManager.instance();
	        blog.setCategories(articleManager.getCategories(userId));
	        blog.setArticles(articleManager.getArticles(userId, article));
	        blog.setAuthors(authorManager.getAuthors(userId));
	        blog.setBookmarks(bookmarkManager.getBookmarks(userId));
        }
        return blog;
    }
    
    public Blog getBlog(String userId, java.util.Date date) {
        Blog blog = getBlogInfo(userId);
        if(blog != null){
	        ArticleManager articleManager = ArticleManager.instance();
	        BlogAuthorManager authorManager = BlogAuthorManager.instance();
	        BlogBookmarkManager bookmarkManager = BlogBookmarkManager.instance();
	        blog.setCategories(articleManager.getCategories(userId));
	        blog.setArticles(articleManager.getArticles(userId, date));
	        blog.setAuthors(authorManager.getAuthors(userId));
	        blog.setBookmarks(bookmarkManager.getBookmarks(userId));
        }
        return blog;
    }
    
    public Blog getBlog(String userId, User author) {
        Blog blog = getBlogInfo(userId);
        if(blog != null){
	        ArticleManager articleManager = ArticleManager.instance();
	        BlogAuthorManager authorManager = BlogAuthorManager.instance();
	        BlogBookmarkManager bookmarkManager = BlogBookmarkManager.instance();
	        blog.setCategories(articleManager.getCategories(userId));
	        blog.setArticles(articleManager.getArticles(userId, author));
	        blog.setAuthors(authorManager.getAuthors(userId));
	        blog.setBookmarks(bookmarkManager.getBookmarks(userId));
        }
        return blog;
    }
    
    public Blog getBlogInfo(String userId) {
        Blog blog = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        StringBuffer sb = new StringBuffer();
        sb.append("SELECT title, description, template, created, totalArticleCount, pageSize, url, allowRSS, titleImage, backgroundImage ");
        sb.append("FROM manalith_blog ");
        sb.append("WHERE owner=?");

        try {
            pstmt = conn.prepareStatement(sb.toString());

            pstmt.setString(1, userId);

            rs = pstmt.executeQuery();

            if (rs.next()) {
                blog = new Blog();
                blog.setTitle(rs.getString("title"));
                blog.setDescription(rs.getString("description"));
                blog.setTemplate(new Template(rs.getString("template")));
                blog.setOwner(new User(userId));
                blog.setCreated(new Date(rs.getTimestamp("created").getTime()));
                blog.setTotalArticleCount(rs.getInt("totalArticleCount"));
                blog.setPageSize(rs.getInt("pageSize"));
                blog.setUrl(rs.getString("url"));
                blog.setAllowRSS(rs.getBoolean("allowRSS"));
                blog.setTitleImage(rs.getString("titleImage"));
                blog.setBackgroundImage(rs.getString("backgroundImage"));
            }
        } catch (SQLException e) {
            logger.error(e.toString());
        }finally {
            if (rs != null) {
                try {
                    rs.close(); 
                } catch (SQLException e) {
                    logger.error(e.toString());
                }
                rs = null; 
            }
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }

        return blog;
    }

    public void updateBlog(Blog blog) {
        PreparedStatement pstmt = null;
        StringBuffer sb = new StringBuffer();
        sb.append("UPDATE manalith_blog ");
        sb.append("SET title=?, description=?, template=?, pageSize=?, url=?, allowRSS=? ");
        sb.append("WHERE owner=?");

        try {
            pstmt = conn.prepareStatement(sb.toString());

            pstmt.setString(1, blog.getTitle());
            pstmt.setString(2, blog.getDescription());
            pstmt.setString(3, blog.getTemplate().toString());
            pstmt.setInt(4, blog.getPageSize());
            pstmt.setString(5, blog.getUrl());
            pstmt.setBoolean(6, blog.getAllowRSS());
            pstmt.setString(7, blog.getOwner().toString());

            pstmt.executeUpdate();
        } catch (SQLException e) {
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }

    //FIXME : 너무나 너무나 많은 관련 테이블 행의 제거 : manalith_blog_category,
    // manalith_blog_author 등
    public void destroyBlog(String userId) {
        PreparedStatement pstmt = null;
        String sql = "DELETE FROM manalith_blog_article WHERE blogOwnerId=?";
        String sql2 = "DELETE FROM manalith_blog WHERE owner=?";

        try {
            pstmt = conn.prepareStatement(sql);
            pstmt.setString(1, userId);
            pstmt.executeUpdate();

            pstmt = conn.prepareStatement(sql2);
            pstmt.setString(1, userId);
            pstmt.executeUpdate();
        } catch (SQLException e) {
            logger.error(e.toString());
        }finally {
            if (pstmt != null) {
                try {
                  pstmt.close(); 
                } catch (SQLException e) {
        	        logger.error(e.toString());
                }
                pstmt = null; 
            } 
        }
    }

}
